@@ -565,7 +565,7 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
         if (var->vartype == ORC_VAR_TYPE_SRC ||
             var->vartype == ORC_VAR_TYPE_DEST) {
           OrcInstruction *cinsn;
-          
+
           cinsn = compiler->insns + compiler->n_insns;
           compiler->n_insns++;
 
@@ -580,10 +580,7 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
         } else if (var->vartype == ORC_VAR_TYPE_CONST ||
             var->vartype == ORC_VAR_TYPE_PARAM) {
           OrcInstruction *cinsn;
-          int multiplier;
-
-          cinsn = compiler->insns + compiler->n_insns;
-          compiler->n_insns++;
+          int l, multiplier, loaded;
 
           multiplier = 1;
           if (insn.flags & ORC_INSTRUCTION_FLAG_X2) {
@@ -593,6 +590,22 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
             multiplier = 4;
           }
 
+          loaded = -1;
+          for(l=0;l<ORC_N_COMPILER_VARIABLES;l++){
+            if (compiler->vars[l].name == NULL) continue;
+            if (!compiler->vars[l].has_parameter) continue;
+            if (compiler->vars[l].parameter != insn.src_args[i]) continue;
+            if (compiler->vars[l].size != opcode->src_size[i] * multiplier) continue;
+            loaded = l;
+            break;
+          }
+          if (loaded != -1) {
+            insn.src_args[i] = loaded;
+            continue;
+          }
+          cinsn = compiler->insns + compiler->n_insns;
+          compiler->n_insns++;
+
           cinsn->flags = insn.flags;
           cinsn->flags |= ORC_INSN_FLAG_ADDED;
           cinsn->opcode = get_loadp_opcode_for_size (opcode->src_size[i]);
@@ -602,9 +615,10 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
             compiler->vars[cinsn->dest_args[0]].flags |=
                 ORC_VAR_FLAG_VOLATILE_WORKAROUND;
           }
+          compiler->vars[cinsn->dest_args[0]].has_parameter = TRUE;
+          compiler->vars[cinsn->dest_args[0]].parameter = insn.src_args[i];
           cinsn->src_args[0] = insn.src_args[i];
           insn.src_args[i] = cinsn->dest_args[0];
-
         }
       }
     }
@@ -905,10 +919,14 @@ orc_compiler_rewrite_vars2 (OrcCompiler *compiler)
      */
     if (compiler->insns[j].flags & ORC_INSN_FLAG_INVARIANT) continue;
 
-    if (!(compiler->insns[j].opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR)
-        && compiler->insns[j].opcode->dest_size[1] == 0) {
+    if (!(compiler->insns[j].opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR)) {
       int src1 = compiler->insns[j].src_args[0];
-      int dest = compiler->insns[j].dest_args[0];
+      int dest;
+
+      if (compiler->insns[j].opcode->dest_size[1] == 0)
+        dest = compiler->insns[j].dest_args[0];
+      else
+        dest = compiler->insns[j].dest_args[1];
 
       if (compiler->vars[src1].last_use == j) {
         if (compiler->vars[src1].first_use == j) {
@@ -927,7 +945,7 @@ orc_compiler_rewrite_vars2 (OrcCompiler *compiler)
       compiler->vars[src2].alloc = 1;
     } else {
       int src2 = compiler->insns[j].src_args[1];
-      if (compiler->vars[src2].alloc == 1) {
+      if (src2 != -1 && compiler->vars[src2].alloc == 1) {
         compiler->vars[src2].alloc = 0;
       }
     }
@@ -1052,25 +1070,26 @@ orc_compiler_get_constant (OrcCompiler *compiler, int size, int value)
 {
   int i;
   int tmp;
+  unsigned int v = value;
 
   if (size < 4) {
     if (size < 2) {
-      value &= 0xff;
-      value |= (value<<8);
+      v &= 0xff;
+      v |= (v<<8);
     }
-    value &= 0xffff;
-    value |= (value<<16);
+    v &= 0xffff;
+    v |= (v<<16);
   }
 
   for(i=0;i<compiler->n_constants;i++){
     if (compiler->constants[i].is_long == FALSE &&
-        compiler->constants[i].value == value) {
+        compiler->constants[i].value == v) {
       break;
     }
   }
   if (i == compiler->n_constants) {
     compiler->n_constants++;
-    compiler->constants[i].value = value;
+    compiler->constants[i].value = v;
     compiler->constants[i].alloc_reg = 0;
     compiler->constants[i].use_count = 0;
     compiler->constants[i].is_long = FALSE;
@@ -1090,36 +1109,14 @@ int
 orc_compiler_get_constant_long (OrcCompiler *compiler,
     orc_uint32 a, orc_uint32 b, orc_uint32 c, orc_uint32 d)
 {
-  int i;
   int tmp;
 
-  for(i=0;i<compiler->n_constants;i++){
-    if (compiler->constants[i].is_long == TRUE &&
-        compiler->constants[i].full_value[0] == a &&
-        compiler->constants[i].full_value[1] == b &&
-        compiler->constants[i].full_value[2] == c &&
-        compiler->constants[i].full_value[3] == d) {
-      break;
-    }
+  tmp = orc_compiler_try_get_constant_long (compiler, a, b, c, d);
+  if (tmp == ORC_REG_INVALID) {
+    tmp = orc_compiler_get_temp_reg (compiler);
+    orc_compiler_load_constant_long (compiler, tmp,
+        &compiler->constants[compiler->n_constants - 1]);
   }
-  if (i == compiler->n_constants) {
-    compiler->n_constants++;
-    compiler->constants[i].full_value[0] = a;
-    compiler->constants[i].full_value[1] = b;
-    compiler->constants[i].full_value[2] = c;
-    compiler->constants[i].full_value[3] = d;
-    compiler->constants[i].is_long = TRUE;
-    compiler->constants[i].alloc_reg = 0;
-    compiler->constants[i].use_count = 0;
-  }
-
-  compiler->constants[i].use_count++;
-
-  if (compiler->constants[i].alloc_reg != 0) {;
-    return compiler->constants[i].alloc_reg;
-  }
-  tmp = orc_compiler_get_temp_reg (compiler);
-  orc_compiler_load_constant_long (compiler, tmp, &compiler->constants[i]);
   return tmp;
 }
 
@@ -1184,6 +1181,9 @@ orc_compiler_get_constant_reg (OrcCompiler *compiler)
       compiler->alloc_regs[compiler->constants[j].alloc_reg] = 1;
     }
   }
+  if (compiler->max_used_temp_reg < compiler->min_temp_reg)
+    compiler->max_used_temp_reg = compiler->min_temp_reg;
+
   for(j=ORC_VEC_REG_BASE;j<=compiler->max_used_temp_reg;j++) {
     compiler->alloc_regs[j] = 1;
   }
