@@ -459,7 +459,7 @@ read_file (const char *filename)
   long size;
   int ret;
 
-  file = fopen (filename, "r");
+  file = fopen (filename, "rb");
   if (file == NULL) return NULL;
 
   ret = fseek (file, 0, SEEK_END);
@@ -537,13 +537,13 @@ static const char *orcify_typename (const char *s)
 }
 
 void
-output_prototype (OrcProgram *p, FILE *output)
+output_prototype (OrcProgram *p, FILE *output, int backup)
 {
   OrcVariable *var;
   int i;
   int need_comma;
 
-  fprintf(output, "%s (", p->name);
+  fprintf(output, "%s (", backup ? p->backup_name : p->name);
   need_comma = FALSE;
   for(i=0;i<4;i++){
     var = &p->vars[ORC_VAR_D1 + i];
@@ -632,6 +632,114 @@ output_prototype (OrcProgram *p, FILE *output)
   fprintf(output, ")");
 }
 
+void
+output_executor_backup_call (OrcProgram *p, FILE *output)
+{
+  OrcVariable *var;
+  int i;
+
+  fprintf(output, "  %s (", p->backup_name);
+  for(i=0;i<4;i++){
+    var = &p->vars[ORC_VAR_D1 + i];
+    if (var->size) {
+      fprintf(output, "ex->arrays[%s], ", enumnames[ORC_VAR_D1 + i]);
+      if (p->is_2d) {
+        fprintf(output, "ex->params[%s], ", enumnames[ORC_VAR_D1 + i]);
+      }
+    }
+  }
+  for(i=0;i<8;i++){
+    var = &p->vars[ORC_VAR_S1 + i];
+    if (var->size) {
+      fprintf(output, "ex->arrays[%s], ", enumnames[ORC_VAR_S1 + i]);
+      if (p->is_2d) {
+        fprintf(output, "  ex->params[%s], ", enumnames[ORC_VAR_S1 + i]);
+      }
+    }
+  }
+  for(i=0;i<8;i++){
+    var = &p->vars[ORC_VAR_P1 + i];
+    if (var->size) {
+      switch (var->param_type) {
+        case ORC_PARAM_TYPE_INT:
+          fprintf(output, "ex->params[%s],", enumnames[ORC_VAR_P1 + i]);
+          break;
+        case ORC_PARAM_TYPE_FLOAT:
+          fprintf(output, "((orc_union32 * )&ex->params[%s])->f, ",
+              enumnames[ORC_VAR_P1 + i]);
+          break;
+        case ORC_PARAM_TYPE_INT64:
+          fprintf(output, "(ex->params[%s] & 0xffffffff) | ((orc_uint64)(ex->params[%s]) << 32), ", enumnames[ORC_VAR_P1 + i], enumnames[ORC_VAR_T1 + i]);
+          break;
+        case ORC_PARAM_TYPE_DOUBLE:
+          /* FIXME */
+          break;
+        default:
+          ORC_ASSERT(0);
+      }
+    }
+  }
+  if (p->constant_n) {
+    fprintf(output, "%d", p->constant_n);
+  } else {
+    fprintf(output, "ex->n");
+  }
+  if (p->is_2d) {
+    if (p->constant_m) {
+      fprintf(output, ",  %d", p->constant_m);
+    } else {
+      fprintf(output, ", ORC_EXECUTOR_M(ex)");
+    }
+  }
+  fprintf(output, ");\n");
+}
+
+void
+output_backup_call (OrcProgram *p, FILE *output)
+{
+  OrcVariable *var;
+  int i;
+
+  fprintf(output, "  %s (", p->backup_name);
+  for(i=0;i<4;i++){
+    var = &p->vars[ORC_VAR_D1 + i];
+    if (var->size) {
+      fprintf(output, "%s, ", varnames[ORC_VAR_D1 + i]);
+      if (p->is_2d) {
+        fprintf(output, "%s_stride, ", varnames[ORC_VAR_D1 + i]);
+      }
+    }
+  }
+  for(i=0;i<8;i++){
+    var = &p->vars[ORC_VAR_S1 + i];
+    if (var->size) {
+      fprintf(output, "%s, ", varnames[ORC_VAR_S1 + i]);
+      if (p->is_2d) {
+        fprintf(output, "%s_stride, ", varnames[ORC_VAR_S1 + i]);
+      }
+    }
+  }
+  for(i=0;i<8;i++){
+    var = &p->vars[ORC_VAR_P1 + i];
+    if (var->size) {
+        fprintf(output, "%s, ", varnames[ORC_VAR_P1 + i]);
+    }
+  }
+  if (p->constant_n) {
+    fprintf(output, "%d", p->constant_n);
+  } else {
+    fprintf(output, "n");
+  }
+  if (p->is_2d) {
+    if (p->constant_m) {
+      fprintf(output, ", %d", p->constant_m);
+    } else {
+      fprintf(output, ", m");
+    }
+  }
+  fprintf(output, ");\n");
+}
+
 void
 output_code_header (OrcProgram *p, FILE *output)
 {
@@ -640,8 +748,13 @@ output_code_header (OrcProgram *p, FILE *output)
   } else {
     fprintf(output, "void ");
   }
-  output_prototype (p, output);
+  output_prototype (p, output, 0);
   fprintf(output, ";\n");
+  if (p->backup_name && mode != MODE_TEST) {
+    fprintf(output, "void ");
+    output_prototype (p, output, 1);
+    fprintf(output, ";\n");
+  }
 }
 
 void
@@ -655,7 +768,9 @@ output_code_backup (OrcProgram *p, FILE *output)
     fprintf(output, "_backup_%s (OrcExecutor * ORC_RESTRICT ex)\n", p->name);
   }
   fprintf(output, "{\n");
-  {
+  if (p->backup_name && mode != MODE_TEST) {
+    output_executor_backup_call (p, output);
+  } else {
     OrcCompileResult result;
 
     result = orc_program_compile_full (p, orc_target_get_by_name("c"),
@@ -677,9 +792,11 @@ output_code_no_orc (OrcProgram *p, FILE *output)
 {
 
   fprintf(output, "void\n");
-  output_prototype (p, output);
+  output_prototype (p, output, 0);
   fprintf(output, "{\n");
-  {
+  if (p->backup_name && mode != MODE_TEST) {
+    output_backup_call (p, output);
+  } else {
     OrcCompileResult result;
 
     result = orc_program_compile_full (p, orc_target_get_by_name("c"),
@@ -740,7 +857,7 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
   } else {
     fprintf(output, "void\n");
   }
-  output_prototype (p, output);
+  output_prototype (p, output, 0);
   fprintf(output, "\n");
   fprintf(output, "{\n");
   fprintf(output, "  OrcExecutor _ex, *ex = &_ex;\n");
@@ -843,9 +960,9 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
           fprintf(output, "  {\n");
           fprintf(output, "    orc_union64 tmp;\n");
           fprintf(output, "    tmp.i = %s;\n", varnames[ORC_VAR_P1 + i]);
-          fprintf(output, "    ex->params[%s] = tmp.x2[0];\n",
+          fprintf(output, "    ex->params[%s] = ((orc_uint64) tmp.i) & 0xffffffff;\n",
               enumnames[ORC_VAR_P1 + i]);
-          fprintf(output, "    ex->params[%s] = tmp.x2[1];\n",
+          fprintf(output, "    ex->params[%s] = ((orc_uint64) tmp.i) >> 32;\n",
               enumnames[ORC_VAR_T1 + i]);
           fprintf(output, "  }\n");
           break;
@@ -854,9 +971,9 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
           fprintf(output, "  {\n");
           fprintf(output, "    orc_union64 tmp;\n");
           fprintf(output, "    tmp.f = %s;\n", varnames[ORC_VAR_P1 + i]);
-          fprintf(output, "    ex->params[%s] = tmp.x2[0];\n",
+          fprintf(output, "    ex->params[%s] = ((orc_uint64) tmp.i) & 0xffffffff;\n",
               enumnames[ORC_VAR_P1 + i]);
-          fprintf(output, "    ex->params[%s] = tmp.x2[1];\n",
+          fprintf(output, "    ex->params[%s] = ((orc_uint64) tmp.i) >> 32;\n",
               enumnames[ORC_VAR_T1 + i]);
           fprintf(output, "  }\n");
           break;
@@ -1203,13 +1320,8 @@ output_code_test (OrcProgram *p, FILE *output)
     var = &p->vars[ORC_VAR_C1 + i];
     if (var->size) {
       if (var->size < 8) {
-        if (var->value.i != 0x80000000) {
-          fprintf(output, "      orc_program_add_constant (p, %d, 0x%08x, \"%s\");\n",
-              var->size, (int)var->value.i, varnames[ORC_VAR_C1 + i]);
-        } else {
-          fprintf(output, "      orc_program_add_constant (p, %d, 0x%08x, \"%s\");\n",
-              var->size, (int)var->value.i, varnames[ORC_VAR_C1 + i]);
-        }
+	fprintf(output, "      orc_program_add_constant (p, %d, 0x%08x, \"%s\");\n",
+	    var->size, (int)var->value.i, varnames[ORC_VAR_C1 + i]);
       } else {
         fprintf(output, "      orc_program_add_constant_int64 (p, %d, "
             "0x%08x%08xULL, \"%s\");\n",
